/* Test of freadable() function.
   Copyright (C) 2007 Free Software Foundation, Inc.

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software Foundation,
   Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  */

/* Written by Bruno Haible <bruno@clisp.org>, 2007.  */

#include <config.h>

#include "freadable.h"

#include <stdio.h>
#include <stdlib.h>

#define ASSERT(expr) \
  do									     \
    {									     \
      if (!(expr))							     \
        {								     \
          fprintf (stderr, "%s:%d: assertion failed\n", __FILE__, __LINE__); \
          abort ();							     \
        }								     \
    }									     \
  while (0)

#define TESTFILE "t-freadable.tmp"

int
main ()
{
  FILE *fp;

  /* Create a file with some contents.  */
  fp = fopen (TESTFILE, "w");
  if (fp == NULL)
    goto skip;
  ASSERT (!freadable (fp));
  if (fwrite ("foobarsh", 1, 8, fp) < 8)
    goto skip;
  ASSERT (!freadable (fp));
  if (fclose (fp))
    goto skip;

  /* Open it in read-only mode.  */
  fp = fopen (TESTFILE, "r");
  if (fp == NULL)
    goto skip;
  ASSERT (freadable (fp));
  if (fgetc (fp) != 'f')
    goto skip;
  ASSERT (freadable (fp));
  if (fseek (fp, 2, SEEK_CUR))
    goto skip;
  ASSERT (freadable (fp));
  if (fgetc (fp) != 'b')
    goto skip;
  ASSERT (freadable (fp));
  fflush (fp);
  ASSERT (freadable (fp));
  if (fgetc (fp) != 'a')
    goto skip;
  ASSERT (freadable (fp));
  if (fseek (fp, 0, SEEK_END))
    goto skip;
  ASSERT (freadable (fp));
  if (fclose (fp))
    goto skip;

  /* Open it in read-write mode.  */
  fp = fopen (TESTFILE, "r+");
  if (fp == NULL)
    goto skip;
  ASSERT (freadable (fp));
  if (fgetc (fp) != 'f')
    goto skip;
  ASSERT (freadable (fp));
  if (fseek (fp, 2, SEEK_CUR))
    goto skip;
  ASSERT (freadable (fp));
  if (fgetc (fp) != 'b')
    goto skip;
  ASSERT (freadable (fp));
  fflush (fp);
  ASSERT (freadable (fp));
  if (fgetc (fp) != 'a')
    goto skip;
  ASSERT (freadable (fp));
  if (fputc ('z', fp) != 'z')
    goto skip;
  ASSERT (freadable (fp));
  if (fseek (fp, 0, SEEK_END))
    goto skip;
  ASSERT (freadable (fp));
  if (fclose (fp))
    goto skip;

  /* Open it in append mode.  */
  fp = fopen (TESTFILE, "a");
  if (fp == NULL)
    goto skip;
  ASSERT (!freadable (fp));
  if (fwrite ("bla", 1, 3, fp) < 3)
    goto skip;
  ASSERT (!freadable (fp));
  if (fclose (fp))
    goto skip;

  return 0;

 skip:
  fprintf (stderr, "Skipping test: file operations failed.\n");
  return 77;
}
